var request = require('superagent');

var getContext = require('../helpers/get-context'),
    getQueryString = require('../helpers/get-query-string'),
    getUrlMaxLength = require('../helpers/get-url-max-length'),
    getXHR = require('../helpers/get-xhr-object'),
    requestTypes = require('../helpers/superagent-request-types'),
    responseHandler = require('../helpers/superagent-handle-response');

module.exports = function(path, params, callback){
  var self = this,
      urlBase = this.client.url(path),
      reqType = this.client.config.requestType,
      cb = callback;

  callback = null;

  if (!self.client.projectId()) {
    self.client.trigger('error', 'Query not sent: Missing projectId property');
    return;
  }

  if (!self.client.readKey()) {
    self.client.trigger('error', 'Query not sent: Missing readKey property');
    return;
  }

  if (getXHR() || getContext() === 'server' ) {
    request
      .post(urlBase)
        .set('Content-Type', 'application/json')
        .set('Authorization', self.client.readKey())
        .timeout(self.timeout())
        .send(params || {})
        .end(handleResponse);
  }
  else {
    extend(params, { api_key: self.client.readKey() });
    urlBase += getQueryString(params);
    if (urlBase.length < getUrlMaxLength() ) {
      request
        .get(urlBase)
        .timeout(self.timeout())
        .use(requestTypes('jsonp'))
        .end(handleResponse);
    }
    else {
      self.client.trigger('error', 'Query not sent: URL length exceeds current browser limit, and XHR (POST) is not supported.');
    }
  }

  function handleResponse(err, res){
    responseHandler(err, res, cb);
    cb = callback = null;
  }

  return;
}
